//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Models/ItemWithParticles.cpp
//! @brief     Implements class ItemWithParticles
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "GUI/Models/ItemWithParticles.h"
#include "GUI/Models/RotationItems.h"
#include "GUI/Models/SessionItemUtils.h"
#include "GUI/Models/SessionModel.h"
#include "GUI/Models/TransformationItem.h"
#include "GUI/Models/VectorItem.h"

const QString ItemWithParticles::P_ABUNDANCE = "Abundance";
const QString ItemWithParticles::P_POSITION = "Position Offset";
const QString ItemWithParticles::T_TRANSFORMATION = "Transformation Tag";

double ItemWithParticles::abundance() const
{
    return getItemValue(P_ABUNDANCE).toDouble();
}

void ItemWithParticles::setAbundance(const double abundance)
{
    setItemValue(P_ABUNDANCE, abundance);
}

SessionItem* ItemWithParticles::abundanceItem() const
{
    return getItem(P_ABUNDANCE);
}

const kvector_t ItemWithParticles::position() const
{
    return item<VectorItem>(P_POSITION)->getVector();
}

void ItemWithParticles::setPosition(const kvector_t& position)
{
    item<VectorItem>(P_POSITION)->setVector(position);
}

VectorItem* ItemWithParticles::positionItem() const
{
    return item<VectorItem>(P_POSITION);
}

TransformationItem* ItemWithParticles::transformation() const
{
    return dynamic_cast<TransformationItem*>(getItem(T_TRANSFORMATION));
}

TransformationItem* ItemWithParticles::createTransformationItem()
{
    return model()->insertItem<TransformationItem>(this, -1, T_TRANSFORMATION);
}

void ItemWithParticles::setTransformation(RotationItem* transformation)
{
    model()->moveItem(transformation, this, -1, T_TRANSFORMATION);
}

bool ItemWithParticles::isTransformationTagName(const QString& name)
{
    return name == T_TRANSFORMATION;
}

ItemWithParticles::ItemWithParticles(const QString& model_type, const QString& abundance_tooltip,
                                     const QString& position_tooltip)
    : SessionGraphicsItem(model_type)
{
    addProperty(P_ABUNDANCE, 1.0)
        ->setLimits(RealLimits::limited(0.0, 1.0))
        .setDecimals(3)
        .setToolTip(abundance_tooltip);
    addProperty<VectorItem>(P_POSITION)->setToolTip(position_tooltip);

    registerTag(T_TRANSFORMATION, 0, 1, {TransformationItem::M_TYPE});

    mapper()->setOnParentChange([this](SessionItem* parent) {
        if (GUI::Session::ItemUtils::HasOwnAbundance(parent)) {
            setItemValue(P_ABUNDANCE, 1.0);
            getItem(P_ABUNDANCE)->setEnabled(false);
            if (isShellParticle()) {
                auto pos = item<VectorItem>(P_POSITION);
                pos->setVector(kvector_t());
                pos->setEnabled(false);
            }
        } else {
            getItem(P_ABUNDANCE)->setEnabled(true);
            if (isShellParticle())
                item<VectorItem>(P_POSITION)->setEnabled(true);
        }
    });
}

void ItemWithParticles::setDefaultTagTransformation()
{
    setDefaultTag(T_TRANSFORMATION);
}

bool ItemWithParticles::isShellParticle() const
{
    return false;
}
